/*--------------------------------------------------------------+
| Author: Daniel Porumbel 2018                                  |
|License: Creative Commons Attribution 3.0 Unported License     |
|         http://creativecommons.org/licenses/by/3.0/           |             
+--------------------------------------------------------------*/
#ifndef SPHEREARCHIVE_H
#define SPHEREARCHIVE_H
#include<iostream>
#include<fstream>
#include<iomanip>
using namespace std;

/*-----------------------+------------------------------------+--------------------------
                         |  Two Hamming distance functions    |
                         +-----------------------------------*/
//simple Hamming distance
int dist(int* sol1, int*sol2);      
//change/delta of dist(current sol, center) *AFTER* move sol[enter]:0-->1,sol[exit]:1-->0
int deltaDist(int enter, int exit, int* center); 


/*-----------------------+------------------------------------+--------------------------
                         |  Parameters of the priority queue  |
                         +-----------------------------------*/

extern int n;                      //size of a solution
extern int maxPerSphere;           //number max of runs launched from each center
extern int runsLaunched;           //number of runs ever launched since the beginning

#define FIFO_RANK_SAME_OBJ_CENTERS //2nd sphere ranking criterion=FIFO, not smthing using distances


/*-----------------------+------------------------------------+--------------------------
                         |   Main priority queue functions    |
                         +-----------------------------------*/

//retrieve the first sphere in the priority queue, put the result in sol (allocated already)
void dequeueMaxPriorCenter(int *sol);
//inserts in the priority queue a solution with objective value objectiveValue
void enqueueCenter(int* newCenter, int objectiveValue);
//print archive for debugging
void printPrioritQueue();
//free memory
void erasePrioritQueue();





/*--------------------------------------------------------------------------------*/
/*------                    Private data, for information only                 ----*/
//#define MDS_LOG
#ifdef MDS_LOG
extern int idxGlob ;
extern ofstream arrows;
extern ofstream mat;
void printMat();
#endif

struct sphere{               //Each sphere is characterized by:
    int obj;                 //obj value of the center
    int sumDst;              //sum of distances to other sphere centers
    int launched;            //number of runs launched from current center (<=maxPerSphere)
    int*sol;                 //the solution itself (eg, 0/1 array for clique, or arrays with more colors for graph coloring
    #ifdef MDS_LOG
    int idx;
    #endif
    sphere* nxt;
};
//return 1 if center is better than a new sphere with obj=objNew and sumDst=sumDstNew
int compCenters(sphere* center, int objNew, int sumDstNew);
#endif
